@@ -1,17 +1,64 @@
+# Copyright (c) 2019,2020 Fabien Georget <fabien.georget@epfl.ch>, EPFL
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
+#
+# 1. Redistributions of source code must retain the above copyright notice,
+# this list of conditions and the following disclaimer.
+#
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+# this list of conditions and the following disclaimer in the documentation
+# and/or other materials provided with the distribution.
+#
+# 3. Neither the name of the copyright holder nor the names of its contributors
+# may be used to endorse or promote products derived from this software without
+# specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
+# IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
+
+"""This module loads the quantified EDS maps into Glue
+"""
+
+
+import os.path
+
+import numpy as np
+
+import matplotlib.pyplot as plt
+from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
+from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
+from matplotlib.figure import Figure
+from matplotlib.gridspec import GridSpec
+
+import qtpy.QtWidgets as qtw
+import qtpy.QtGui as qtg
+import qtpy.QtCore as qtc
+
 from glue.config import importer
 from glue.utils.qt.helpers import load_ui
 from glue.core.coordinates import AffineCoordinates
 
+## edxia import
 
 from ...core.experiment import MappingExperiment
 
 from edxia.filters import denoise
 
 from ...filters.denoise import DenoiseFilter, UniformFilter, GaussianFilter
-from ...io.loader import PickleLoader, DefaultLoader, StackLoader
+from ...io.loader import DefaultLoader, StackLoader
 from ...composite import CompositeChannels
 from ...composite.segmentation import SlicSegmenter
-#from ...glue.map_coordinates import MapCoordinates
 from ...point_analysis.points import points_from_segmentation
 
 from ...io.raw_io import TextMapFormat
@@ -20,17 +67,9 @@ from ...io.hdf5 import save_dataset, read_dataset
 
 from .. import gluedata_from_points, gluedata_from_stack_and_composite, gluedata_from_stack
 
-import numpy as np
 
-import matplotlib.pyplot as plt
-from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
-from matplotlib.figure import Figure
-from matplotlib.gridspec import GridSpec
-import qtpy.QtWidgets as qtw
-import qtpy.QtGui as qtg
-import qtpy.QtCore as qtc
 
-import os.path
+
 
 
 class PreviewBSE(FigureCanvas):
@@ -94,6 +133,9 @@ class LoadEDSdata(qtw.QDialog):
 
         ui.reload_preview_button.clicked.connect(self.format_has_changed)
 
+        ui.test_filter_button.clicked.connect(self.test_filter)
+        ui.test_segmentation_button.clicked.connect(self.test_segmentation)
+
         self.set_preview_canvas()
         self.results = []
 
@@ -246,7 +288,7 @@ class LoadEDSdata(qtw.QDialog):
     def set_combo_box(self):
         """Set the combo boxes in parameters."""
         ui = self._ui
-        for combo in [ui.green_combo, ui.red_combo, ui.blue_combo]:
+        for combo in [ui.green_combo, ui.red_combo, ui.blue_combo, ui.test_filter_combo]:
             combo.clear()
             combo.addItem("None")
             combo.addItems(self.preview_exp.list_components)
@@ -279,7 +321,11 @@ class LoadEDSdata(qtw.QDialog):
                 ui.radius_filter_box,
                 ui.sigma_filter_box,
 
+                ui.test_filter_button,
+                ui.test_filter_combo,
+
                 ui.segmentation_check,
+                ui.test_segmentation_button,
 
                 ui.green_combo,
                 ui.red_combo,
@@ -320,19 +366,62 @@ class LoadEDSdata(qtw.QDialog):
         """Return the filter to load the maps."""
         if self._ui.tv_filter_check.isChecked():
             weight = float(self._ui.weight_filter_box.text())
-            return [DenoiseFilter(weight),]
+            name = "TotalVariation({0})".format(weight)
+            filters = [DenoiseFilter(weight),]
         elif self._ui.mean_filter_check.isChecked():
             radius = self._ui.radius_filter_box.value()
-            return [UniformFilter(radius),]
+            name = "UniformFilter({0})".format(radius)
+            filters = [UniformFilter(radius),]
         elif self._ui.gaussian_filter_check.isChecked():
             sigma = self._ui.sigma_filter_box.value()
-            return [GaussianFilter(sigma),]
+            name = "GaussianFilter({0})".format(sigma)
+            filters = [GaussianFilter(sigma),]
         elif self._ui.joint_filter_check.isChecked():
             sigmaR = self._ui.sigmaR_filter_box.value()
             sigmaS = self._ui.sigmaS_filter_box.value()
-            return [denoise.CVJointBilateralFilter(sigmaS, sigmaR, exp),]
+            name = "JointBilateralFilter({0},{1})".format(sigmaR,sigmaS)
+            filters = [denoise.CVJointBilateralFilter(sigmaS, sigmaR, exp),]
         else:
-            return None
+            name = "NoFilter()"
+            filters = None
+        return filters, name
+
+
+    def test_filter(self):
+        """Test the filter on a chosen map"""
+        qtw.QApplication.setOverrideCursor(qtg.QCursor(qtc.Qt.WaitCursor))
+        try:
+            self._test_filter()
+        finally:
+            qtw.QApplication.restoreOverrideCursor()
+
+    def _test_filter(self):
+        """Test the filter on a chosen map"""
+        exp =self.preview_exp
+
+        filters, name_filter = self.get_filters(exp)
+        input_loader = DefaultLoader(exp, filters=filters)
+
+        comp = self._ui.test_filter_combo.currentText()
+        if comp == "None":
+            return
+        filtered_map = input_loader.load_edsmap(comp)
+
+        dialog = qtw.QDialog(self)
+        dialog.setModal(False)
+
+        canvas = TestFilterCanvas(filtered_map.map)
+        toolbar = NavigationToolbar(canvas, self)
+
+        layout = qtw.QVBoxLayout()
+        layout.addWidget(toolbar)
+        layout.addWidget(canvas)
+
+        dialog.setLayout(layout)
+        dialog.setWindowTitle(name_filter)
+        dialog.show()
+
+
 
     def get_scale(self):
         """Return the coordinate system"""
@@ -369,11 +458,50 @@ class LoadEDSdata(qtw.QDialog):
         return (self._ui.unit_pwt_button.isChecked()
                 or self._ui.unit_wt_button.isChecked())
 
+    def test_segmentation(self):
+        qtw.QApplication.setOverrideCursor(qtg.QCursor(qtc.Qt.WaitCursor))
+        try:
+            self._test_segmentation()
+        finally:
+            qtw.QApplication.restoreOverrideCursor()
+
+    def _test_segmentation(self):
+        ui = self._ui
+
+        exp =self.preview_exp
+
+        filters, _ = self.get_filters(exp)
+        loader = DefaultLoader(exp, filters=filters)
+        channels = self.get_channels()
+        composite = loader.load_composite(channels)
+        bse_map =  loader.load_edsmap("BSE")
+
+        if ui.bsemix_check.isChecked():
+            composite.mix_with_bse(bse_map, ui.alpha_box.value())
+        compactness = float(ui.compactness_box.text())
+        numberpts = float(ui.numberpts_box.text())
+        labels = SlicSegmenter(compactness, numberpts).apply(composite)
+
+        dialog = qtw.QDialog(self)
+        dialog.setModal(False)
+
+        canvas = TestSegmentationCanvas(composite, labels)
+        toolbar = NavigationToolbar(canvas, self)
+
+        layout = qtw.QVBoxLayout()
+        layout.addWidget(toolbar)
+        layout.addWidget(canvas)
+
+        dialog.setLayout(layout)
+        dialog.setWindowTitle("Segmentation test")
+        dialog.show()
+
+
     def set_data(self):
         ui = self._ui
 
         exp = self.get_experiment()
-        filters = self.get_filters(exp)
+        filters, _ = self.get_filters(exp)
         input_loader = DefaultLoader(exp, filters=filters)
         stack = input_loader.load_stack()
         loader = StackLoader(stack)
@@ -442,7 +570,7 @@ class LoadEDSdata(qtw.QDialog):
         qtw.QApplication.restoreOverrideCursor()
         return super().accept()
 
-@importer("Import EDS/BSE maps")
+@importer("edxia: Import EDS/BSE maps")
 def load_maps():
     """Load maps in glue"""
     dialog = LoadEDSdata()
@@ -453,7 +581,7 @@ def load_maps():
 
     return dialog.results
 
-@importer("Import EDS/BSE maps (dataset)")
+@importer("edxia: Import EDS/BSE maps (dataset)")
 def load_dataset():
     """Load maps in glue"""
     full_dset_path, _ = qtw.QFileDialog.getOpenFileName(None, "Open dataset", ".","hdf5 files (*.hdf5 *.h5);;All Files (*)")
@@ -473,4 +601,35 @@ def load_dataset():
     if points is not None:
         results.append(gluedata_from_points(exp.label+"points", points))
 
-    return results
\ No newline at end of file
+    return results
+
+class TestFilterCanvas(FigureCanvas):
+    """The matplotlib canvas for the test-filter dialog"""
+    def __init__(self, npmap, parent=None):
+
+        fig = Figure(constrained_layout=True)
+
+        super().__init__(fig)
+        if parent is not None:
+            self.setParent(parent)
+
+        self.ax = fig.subplots()
+        mappable = self.ax.imshow(npmap, cmap=plt.cm.gray)
+        fig.colorbar(mappable)
+
+class TestSegmentationCanvas(FigureCanvas):
+    """The matplotlib canvas for the test-segmentation dialog"""
+    def __init__(self, composite, labels, parent=None):
+
+        fig = Figure(constrained_layout=True)
+
+        super().__init__(fig)
+        if parent is not None:
+            self.setParent(parent)
+
+        self.ax1, self.ax2 = fig.subplots(1,2)
+
+        self.ax1.imshow(composite.map)
+        self.ax2.imshow(labels.label2rgb())
+        self.ax1.set_title("Composite image")
+        self.ax2.set_title("Segmented composite")
\ No newline at end of file
