@@ -53,9 +53,10 @@ from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
 from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
 from matplotlib.figure import Figure
 
-from edxia.analysis.helpers import find_image_datasets
+from edxia.analysis.helpers import find_image_datasets, find_points_datasets
 from edxia.analysis.correlation import two_point_correlation_function, lineal_path
 from edxia.analysis.point_counting import multiple_counts
+from edxia.utils.qt import NumpyTableModel
 
 class QuantifySubsets(qtw.QDialog):
     """The main dialog to quantify all the subsets, should be replaced by a submenu if possible"""
@@ -72,6 +73,7 @@ class QuantifySubsets(qtw.QDialog):
         self._ui.s2_correlation_button.clicked.connect(self.run_s2_correlation)
         self._ui.chord_button.clicked.connect(self.run_chord_lineal)
         self._ui.point_fraction_button.clicked.connect(self.run_point_fraction)
+        self._ui.composition_button.clicked.connect(self.run_composition)
 
     def run_surface_fraction(self):
         dialog = SurfaceFractionQuantifier(self.session, self.dc)
@@ -93,7 +95,13 @@ class QuantifySubsets(qtw.QDialog):
         dialog.exec_()
         return
 
-@menubar_plugin("Analyse phase subsets")
+    def run_composition(self):
+        dialog = CompositionQuantifier(self.session, self.dc)
+        dialog.exec_()
+        return
+
+
+@menubar_plugin("edxia: Analyse phase subsets")
 def subset_quantifier(session, data_collection):
     """The menubar action to analyse the subsets"""
     dialog = QuantifySubsets(session, data_collection)
@@ -249,7 +257,56 @@ class SurfaceFractionQuantifier(qtw.QDialog, TableMixin):
         """Fill the table widget."""
         datas_labels, subset_labels, surface_fractions = self.compute_surface_fractions()
 
-        self.model = TableModel(subset_labels, datas_labels, surface_fractions)
+        self.model = NumpyTableModel(subset_labels, datas_labels, surface_fractions, factor=100)
+        self.table.setModel(self.model)
+
+
+
+###################
+#   Composition   #
+###################
+
+class CompositionQuantifier(qtw.QDialog, TableMixin):
+    """Compute and display the composition (average and standard deviation),
+    for a chosen subset."""
+    def __init__(self, session, data_collection, parent=None):
+        super().__init__(parent)
+        self._ui = load_ui('composition.ui', self,
+            directory=os.path.join(os.path.dirname(__file__), "quantifier"))
+
+        self.set_event_filter()
+
+        self.session = session
+        self.dc = data_collection
+
+        for subset in self.dc.subset_groups:
+            self._ui.subset_box.addItem(subset.label, subset)
+        for data in self.dc.data:
+            self._ui.data_box.addItem(data.label, data)
+
+        self._ui.compute_button.clicked.connect(self.compute_composition)
+
+        self.model = None
+
+    def compute_composition(self):
+        """Compute the average and standard deviations of the components."""
+        subset_grp = self._ui.subset_box.currentData()
+        subset_state = subset_grp.subset_state
+        data = self._ui.data_box.currentData()
+
+        indexes = subset_state.to_index_list(data)
+
+        col_labels = ["E", "sigma"]
+        row_labels = []
+        npdata = np.zeros((len(data.components), 2))
+
+        for ind, comp in enumerate(data.components):
+            row_labels.append(comp.label)
+            npa = data[comp].flat[indexes]
+            npdata[ind,0] = npa.mean()
+            npdata[ind,1] = npa.std()
+
+        self.model = NumpyTableModel(row_labels, col_labels, npdata, fmt=".3f")
         self.table.setModel(self.model)
 
 
@@ -667,7 +724,7 @@ class PointFractions(qtw.QDialog, TableMixin):
         column_labels = []
         row_labels = ["<Pi/Pn>", "stdev", "Si/Sn"]
 
-        self.model = TableModel(row_labels, column_labels, data)
+        self.model = NumpyTableModel(row_labels, column_labels, data, factor=100)
         self.table.setModel(self.model)
 
     def fill_combo_boxes(self):
@@ -738,67 +795,3 @@ class PointFractions(qtw.QDialog, TableMixin):
 
 
 
-###########
-# Helpers #
-###########
-
-
-class TableModel(qtc.QAbstractTableModel):
-    """Simple model for numpy numerical data to be used in QTableView"""
-    def __init__(self, row_labels, column_labels, data, parent=None):
-        """
-        :param row_labels: headers for the rows
-        :param column_labels: headers for the columns
-        :param data: the numpy array to display
-        """
-        super().__init__(parent)
-
-        self._row_labels = row_labels
-        self._column_labels = column_labels
-        self._data = data
-
-    def rowCount(self, parent=None):
-        return len(self._row_labels)
-
-    def columnCount(self, parent=None):
-        return len(self._column_labels)
-
-    def data(self, index, role=qtc.Qt.DisplayRole):
-        if index.isValid():
-            if role == qtc.Qt.DisplayRole:
-                return "{0:.2f}".format(self._data[index.row(),index.column()]*100)
-        return None
-
-    def appendColumn(self, name, new_col=None):
-        """Add a new column to the end of the table
-
-        :param name: the header of the column
-        :param new_col: (optional) the new column
-        """
-        ncol = self._data.shape[1]
-        if new_col is None:
-            new_col = np.zeros(self._data.shape[0], 1)
-        if new_col.shape[0] != self._data.shape[0]:
-            raise ValueError("Inconsistent shape for the columns '{0} != {1}'".format(
-                   new_col.shape[0],  self._data.shape[0]))
-        self.beginInsertColumns(qtc.QModelIndex(),ncol,ncol)
-        self._column_labels.append(name)
-        if self._data.shape[1] == 0:
-            self._data = new_col
-        else:
-            self._data = np.hstack((self._data,  new_col))
-        self.endInsertColumns()
-
-    def headerData(self, rowcol, orientation, role=qtc.Qt.DisplayRole):
-        if orientation == qtc.Qt.Horizontal and role == qtc.Qt.DisplayRole:
-            return self._column_labels[rowcol]
-        if orientation == qtc.Qt.Vertical and role == qtc.Qt.DisplayRole:
-            return self._row_labels[rowcol]
-        return None
-
-    def flags(self, index):
-        flags = super(self.__class__, self).flags(index)
-        flags |= qtc.Qt.ItemIsSelectable
-        flags |= qtc.Qt.ItemIsEnabled
-        flags |= qtc.Qt.ItemIsDragEnabled
-        return flags
